@@ -19,6 +19,7 @@
 #include <gdk/gdkx.h>
 
 #include "cddb-slave.h"
+#include "cddb-slave-private.h"
 
 #define CDDBSLAVE_IID "OAFIID:GNOME_Media_CDDBSlave2_Factory"
 #define CDDB_SERVER "freedb.freedb.org"
@@ -27,8 +28,6 @@
 static int running_objects = 0;
 static GConfClient *client = NULL;
 
-gboolean cddb_debugging = FALSE;
-
 enum {
 	CDDB_SEND_FAKE_INFO,
 	CDDB_SEND_REAL_INFO,
@@ -41,6 +40,7 @@ cddb_destroy_cb (GObject *cddb,
 {
 	running_objects--;
 
+	cs_debug ("main.c: destroy_cb: running objects now %d, refcount of slave %d", running_objects, cddb->ref_count);
 	if (running_objects <= 0) {
 		bonobo_main_quit ();
 	}
@@ -57,20 +57,19 @@ get_hostname (void)
 		return g_strdup (name);
 	}
 }
-	
+
 static BonoboObject *
 factory_fn (BonoboGenericFactory *factory,
 	    const char *component_id,
 	    void *closure)
 {
 	CDDBSlave *cddb;
-	gboolean auth;
 	char *server;
 	int port;
-	int info, server_type;
+	int info;
 	char *name;
 	char *hostname;
-	
+
 	/* Get GConf db */
 	if (client == NULL) {
 		client = gconf_client_get_default ();
@@ -101,13 +100,14 @@ factory_fn (BonoboGenericFactory *factory,
 		break;
 	}
 
-	server = gconf_client_get_string (client, 
+	server = gconf_client_get_string (client,
 					  "/apps/CDDB-Slave2/server", NULL);
 	port = gconf_client_get_int (client, "/apps/CDDB-Slave2/port", NULL);
 	g_object_unref (client);
 	client = NULL;
 
 	/* Create the new slave */
+	cs_debug ("main.c: calling cddb_slave_new");
 	cddb = cddb_slave_new (server, port, name, hostname);
 	g_free (server);
 	g_free (name);
@@ -120,6 +120,7 @@ factory_fn (BonoboGenericFactory *factory,
 
 	/* Keep track of our objects */
 	running_objects++;
+	cs_debug ("main.c: running objects now %d", running_objects);
 	g_signal_connect (G_OBJECT (cddb), "destroy",
 			  G_CALLBACK (cddb_destroy_cb), NULL);
 
@@ -145,16 +146,17 @@ cddbslave_init (gpointer data)
 
 		bonobo_running_context_auto_exit_unref (BONOBO_OBJECT (factory));
 		bonobo_main ();
+		cs_debug ("main.c: returned from bonobo_main ()");
+		return TRUE;
 	}
 
 	return FALSE;
 }
 
-int 
+int
 main (int argc,
       char **argv)
 {
-	CORBA_ORB orb;
 	char *cddbdir;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
@@ -165,8 +167,8 @@ main (int argc,
 			    argc, argv, NULL);
 
 	if (g_getenv ("CDDB_SLAVE_DEBUG")) {
-		cddb_debugging = TRUE;
-		g_print ("CDDBSlave2 debugging turned on.\n");
+		cs_set_debug (TRUE);
+		cs_debug ("CDDBSlave2 debugging turned on.");
 	}
 	cddbdir = g_build_filename (g_get_home_dir (),
 				    ".cddbslave", NULL);
@@ -178,8 +180,10 @@ main (int argc,
 		g_error ("~/.cddbslave needs to be a directory");
 	}
 	g_free (cddbdir);
-	
-	cddbslave_init (NULL);
 
+	if (! cddbslave_init (NULL))
+		g_warning ("Could not initialize CDDB Slave");
+
+        cs_debug ("main.c: done.");
 	exit (0);
 }
