@@ -36,10 +36,14 @@ static char *cd_option_device = NULL;
 
 static gboolean cd_option_unique = FALSE;
 static gboolean cd_option_play = FALSE;
+static gboolean cd_option_tray = FALSE;
 
 /* Popup Menu*/
 static GtkWidget *popup_menu = NULL;
 
+/* Gconf ID */
+static guint volume_notify_id; 
+
 /* if env var GNOME_CD_DEBUG is set,
  * g_warning the given message, and if there was an error, the error message
  */
@@ -54,6 +58,21 @@ gcd_warning (const char *message,
 	g_warning (message, error ? error->message : "(None)");
 }
 
+void
+gcd_error (const gchar *message,
+	   GError *error)
+{
+	GtkWidget *dlg;
+
+	gcd_warning (message, error);
+
+	dlg = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_ERROR,
+				      GTK_BUTTONS_CLOSE, message,
+				      error ? error->message : "(None)");
+	gtk_dialog_run (GTK_DIALOG (dlg));
+	gtk_widget_destroy (dlg);
+}
+
 /* if env var GNOME_CD_DEBUG is set, g_print the given message */
 void
 gcd_debug (const gchar *format,
@@ -118,7 +137,7 @@ skip_to_track (GtkWidget *item,
 	int track, end_track;
 	GnomeCDRomStatus *status = NULL;
 	GnomeCDRomMSF msf, *endmsf;
-	GError *error;
+	GError *error = NULL;
 	
 	track = gtk_option_menu_get_history (GTK_OPTION_MENU (gcd->tracks));
 
@@ -138,8 +157,13 @@ skip_to_track (GtkWidget *item,
 	msf.frame = 0;
 
 	if (gcd->cdrom->playmode == GNOME_CDROM_SINGLE_TRACK) {
-		end_track = track + 2;
-		endmsf = &msf;
+		if (gcd->disc_info && (track + 1) >= gcd->disc_info->ntracks) {
+			end_track = -1;
+			endmsf = NULL;
+		} else {	
+			end_track = track + 2;
+			endmsf = &msf;
+		}
 	} else {
 		end_track = -1;
 		endmsf = NULL;
@@ -322,6 +346,8 @@ window_destroy_cb (GtkWidget *window,
 	/* And the track editor */
 	destroy_track_editor ();
 	
+	/* Remove the gconf volume notify*/
+	gconf_client_notify_remove (client, volume_notify_id);
 	g_object_unref (gcd->cdrom);
 	bonobo_main_quit ();
 }
@@ -333,13 +359,14 @@ struct _MenuItem {
 };
 
 struct _MenuItem menuitems[] = {
-	{N_("P_revious track"), GNOME_CD_PREVIOUS, G_CALLBACK (back_cb)},
-	{N_("_Stop"), GNOME_CD_STOP, G_CALLBACK (stop_cb)},
-	{N_("_Play / Pause"), GNOME_CD_PLAY, G_CALLBACK (play_cb)},
-	{N_("_Next track"), GNOME_CD_NEXT, G_CALLBACK (next_cb)},
+	{N_("_Play / Pause"), GTK_STOCK_MEDIA_PLAY, G_CALLBACK (play_cb)},
+	{N_("_Stop"), GTK_STOCK_MEDIA_STOP, G_CALLBACK (stop_cb)},
+	{N_("P_revious"), GTK_STOCK_MEDIA_PREVIOUS, G_CALLBACK (back_cb)},
+	{N_("_Next"), GTK_STOCK_MEDIA_NEXT, G_CALLBACK (next_cb)},
 	{N_("_Eject disc"), GNOME_CD_EJECT, G_CALLBACK (eject_cb)},
 	{N_("_Help"), GTK_STOCK_HELP, G_CALLBACK (help_cb)},
-	{N_("_About CD player"), GNOME_STOCK_ABOUT, G_CALLBACK (about_cb)},
+	{N_("_About"), GTK_STOCK_ABOUT, G_CALLBACK (about_cb)},
+	{N_("_Quit"), GTK_STOCK_QUIT, G_CALLBACK (window_destroy_cb)},
 	{NULL, NULL, NULL}
 };
 
@@ -351,58 +378,64 @@ popup_menu_detach (GtkWidget *attach_widget,
 }
 
 void
-make_popup_menu (GnomeCD *gcd, GdkEventButton *event)
+make_popup_menu (GnomeCD *gcd, GdkEventButton *event, gboolean iconify)
 {
-	int i;
+	int i, no_of_menu_items;
 	GnomeCDRomStatus *status = NULL;
 
-    if (gnome_cdrom_get_status (GNOME_CDROM (gcd->cdrom), &status, NULL) == FALSE) {
-    }
+	if (gnome_cdrom_get_status (GNOME_CDROM (gcd->cdrom), &status, NULL) == FALSE) {
+	}
 	
 	if (!GTK_WIDGET_REALIZED (gcd->display)) {
-       	goto cleanup;
-    }
-	if (popup_menu) {
+		if (!iconify)
+		       	goto cleanup;
+    	}
+
+	if (popup_menu) 
        	gtk_widget_destroy (popup_menu);
-    }
 	
 	popup_menu = gtk_menu_new ();
 	gtk_menu_attach_to_widget (GTK_MENU (popup_menu),
                                GTK_WIDGET (gcd->display),
                                popup_menu_detach);
+	if (iconify == TRUE)
+		no_of_menu_items = 6;
+	else
+		no_of_menu_items = 7;
 	
-	for (i = 0; menuitems[i].name != NULL; i++) {
+	for (i = 0; i <= no_of_menu_items; i++) {
 		GtkWidget *item, *image;
-	    gchar *icon_name;
+		gchar *icon_name;
 
+		/* Add Quit menu item if iconify popup */
+		if (i == 6 && iconify)
+			i = 7;
+		
 		item = gtk_image_menu_item_new_with_mnemonic (_(menuitems[i].name));
 	        
 		/* If status is play, display the pause icon else display the play icon */
-        if (status->audio == GNOME_CDROM_AUDIO_PLAY && i == 2) {
-            icon_name = GNOME_CD_PAUSE;
-		} else {
-          icon_name = menuitems[i].icon;
-		}
-	    
+        	if (status->audio == GNOME_CDROM_AUDIO_PLAY && i == 0) 
+	        	icon_name = GTK_STOCK_MEDIA_PAUSE;
+		else 
+		        icon_name = menuitems[i].icon;
+		
 		if (icon_name != NULL) {
-	        char *ext = strrchr (icon_name, '.');
-	        if (ext == NULL) {
-	           image = gtk_image_new_from_stock (icon_name,
-	    	                             GTK_ICON_SIZE_MENU);
-	        } else {
-	          char *fullname;
-	          fullname = gnome_program_locate_file (NULL,
-              		     GNOME_FILE_DOMAIN_PIXMAP, icon_name,
-                	     TRUE, NULL);
-              if (fullname != NULL) {
-                  image = gtk_image_new_from_file (fullname);
-              } else {
-                image = NULL;
-               	}
-              
-			  g_free (fullname);
-	     	}
-        }
+	        	char *ext = strrchr (icon_name, '.');
+
+		        if (ext == NULL) 
+				image = gtk_image_new_from_stock (icon_name, GTK_ICON_SIZE_MENU);
+		        else {
+				char *fullname;
+				fullname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
+				                                      icon_name, TRUE, NULL);
+				if (fullname != NULL) 
+					image = gtk_image_new_from_file (fullname);
+				else 
+       					image = NULL;
+        	       	       
+				g_free (fullname);
+		     	}
+        	}
 		
 		if (image != NULL) {
 			gtk_widget_show (image);
@@ -410,59 +443,107 @@ make_popup_menu (GnomeCD *gcd, GdkEventButton *event)
 		}
 
 		if (status->cd == GNOME_CDROM_STATUS_OK) {
-            /* Previous menuitem is sensitive when it is  not the first track and
-               we're playing */
-            if (i == 0) {
-                if (status->track <= 1 && 
-					 (status->audio == GNOME_CDROM_AUDIO_STOP || status->audio ==GNOME_CDROM_AUDIO_COMPLETE)) {
-                    gtk_widget_set_sensitive (item, FALSE);
-                } else {
-                  gtk_widget_set_sensitive (item, TRUE);
-                }
-            }
-            /* Next menuitem is sensitive when this is not the last track */
-            if (i == 3) {
-                if (gcd->disc_info && status->track >= gcd->disc_info->ntracks) {
-                    gtk_widget_set_sensitive (item, FALSE);
-                } else {
-                 gtk_widget_set_sensitive (item, TRUE);
-                }
-            }
-        }
+			/* Previous menuitem is sensitive when it is
+			 * not the first track and we're playing */
+	        	if (i == 2) {
+	                	if (status->track <= 1 ||
+				   (status->audio == GNOME_CDROM_AUDIO_STOP ||
+				    status->audio == GNOME_CDROM_AUDIO_COMPLETE)) 
+					gtk_widget_set_sensitive (item, FALSE);
+				else 
+					gtk_widget_set_sensitive (item, TRUE);
+	            	}
+
+		        /* Next menuitem is sensitive when this is not
+			 * the last track */
+			if (i == 3) {
+				if (gcd->disc_info && status->track >= gcd->disc_info->ntracks) 
+					gtk_widget_set_sensitive (item, FALSE);
+				else
+					gtk_widget_set_sensitive (item, TRUE);
+	                }
+	        }
 		
 		/* Disable the first four menu items when no disc is inserted
 		 * Update as per the buttons on the main window
 		 */
-		if (status->cd == GNOME_CDROM_STATUS_NO_DISC && i < 4) {
-	       	gtk_widget_set_sensitive (item, FALSE);
-        }
-
+		if ((status->cd == GNOME_CDROM_STATUS_NO_DISC || status->cd == GNOME_CDROM_STATUS_EMPTY_DISC) && i < 4) 
+			gtk_widget_set_sensitive (item, FALSE);
+        
 		gtk_widget_show (item);
 
 		gtk_menu_shell_append ((GtkMenuShell *)(popup_menu), item);
-		if (menuitems[i].callback != NULL) {
-			g_signal_connect (G_OBJECT (item), "activate",
-	        G_CALLBACK (menuitems[i].callback), gcd);
+		if (i == 4) {
+			GtkWidget *s = gtk_separator_menu_item_new ();
+			gtk_menu_shell_append ((GtkMenuShell *)(popup_menu), s);
+			gtk_widget_show (s);
 		}
+
+		if (menuitems[i].callback != NULL) 
+			g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (menuitems[i].callback), gcd);
+		
 	}
 	
 	if (event != NULL) {
-    	gtk_menu_popup (GTK_MENU (popup_menu),
-    	                NULL, NULL,
-    	                NULL, NULL,
-    	                event->button,
-    	                event->time);
+	   	gtk_menu_popup (GTK_MENU (popup_menu),
+	    	                NULL, NULL,
+	    	                NULL, NULL,
+	    	                event->button,
+	    	                event->time);
    	} else {
-      gtk_menu_popup (GTK_MENU (popup_menu),
-                      NULL, NULL,
-                      NULL, NULL,
-        	          0,
-    	              gtk_get_current_event_time ());
-    }
-    cleanup:
+		gtk_menu_popup (GTK_MENU (popup_menu),
+				NULL, NULL,
+				NULL, NULL,
+				0, gtk_get_current_event_time ());
+    	}
+cleanup:
         g_free (status);
 }
 
+static void
+set_volume (GnomeCD *gcd)
+{
+	GError *error = NULL;
+	GtkTooltips *volume_level_tooltip;
+	gchar *volume_level;
+	gint scaled_volume;
+	double volume;
+	
+	scaled_volume = gconf_client_get_int (client, "/apps/gnome-cd/volume", NULL);
+	/* On error set the Key */	
+	if (scaled_volume < 0) {
+		scaled_volume = 0;
+		gconf_client_set_int (client, "/apps/gnome-cd/volume", 0, NULL);
+	}	
+	else if (scaled_volume > 100) {
+		scaled_volume = 100;
+		gconf_client_set_int (client, "/apps/gnome-cd/volume", 100, NULL);
+	}
+	volume = ((double)scaled_volume/100)*255;
+        
+	if (gnome_cdrom_set_volume (gcd->cdrom, (int) volume, &error) == FALSE) {
+		gcd_warning ("Error setting volume: %s", error);
+		g_error_free (error);
+	}
+	
+	/* Set the value in the slider */	
+	gtk_range_set_value (GTK_RANGE (gcd->slider), volume);
+	/* Set the tooltip */
+	volume_level = g_strdup_printf (_("Volume %d%%"), scaled_volume);
+	gtk_tooltips_set_tip (gcd->tooltips, GTK_WIDGET (gcd->slider), volume_level, NULL);
+	g_free (volume_level);
+}
+
+static void
+volume_changed_cb (GConfClient *_client,
+		guint cnxn_id,
+		GConfEntry *entry,
+		gpointer user_data)
+{
+	GnomeCD *gcd = user_data;
+	set_volume (gcd);
+}
+
 static void
 show_error (GtkWidget	*dialog,
 	    GError 	*error,
@@ -508,7 +589,6 @@ static GnomeCD *
 init_player (const char *device_override)
 {
 	GnomeCD *gcd;
-	GnomeCDRomStatus *status;
 	GtkWidget *display_box;
 	GtkWidget *top_hbox, *button_hbox, *option_hbox;
 	GtkWidget *button;
@@ -660,9 +740,6 @@ init_player (const char *device_override)
 	gtk_range_set_inverted(GTK_RANGE(gcd->slider), TRUE);
 	gtk_box_pack_start (GTK_BOX (top_hbox), gcd->slider, FALSE, FALSE, 0);
 	
-	gtk_tooltips_set_tip (gcd->tooltips, GTK_WIDGET (gcd->slider),
-			      _("Volume control"), NULL);
-
 	gtk_box_pack_start (GTK_BOX (gcd->vbox), top_hbox, TRUE, TRUE, 0);
 	
 	
@@ -687,14 +764,6 @@ init_player (const char *device_override)
 	option_hbox = gtk_hbox_new (FALSE, 2);
 
 	/* Create app controls */
-	button = make_button_from_stock (gcd, GTK_STOCK_INDEX,
-					 G_CALLBACK(open_track_editor),
-					 _("Open track editor"),
-					 _("Track editor"));
-	gtk_widget_set_sensitive (button, FALSE);
-	gtk_box_pack_start (GTK_BOX (option_hbox), button, FALSE, FALSE, 0);
-	gcd->trackeditor_b = button;
-
 	button = make_button_from_stock (gcd, GTK_STOCK_PREFERENCES,
 					 G_CALLBACK(open_preferences),
 					 _("Open preferences"),
@@ -710,41 +779,35 @@ init_player (const char *device_override)
 	gnome_cd_build_track_list_menu (gcd);
 	gtk_box_pack_start (GTK_BOX (option_hbox), gcd->tracks, TRUE, TRUE, 0);
 
+	button = make_button_from_stock (gcd, GTK_STOCK_INDEX,
+					 G_CALLBACK(open_track_editor),
+					 _("Open track editor"),
+					 _("Track editor"));
+	gtk_widget_set_sensitive (button, FALSE);
+	gtk_box_pack_start (GTK_BOX (option_hbox), button, FALSE, FALSE, 0);
+	gcd->trackeditor_b = button;
+
 	gtk_box_pack_start (GTK_BOX (gcd->vbox), option_hbox, FALSE, FALSE, 0);
 	
-	/* Get the initial volume */
-	if (gnome_cdrom_get_status (gcd->cdrom, &status, NULL) == TRUE) {
-		gtk_range_set_value (GTK_RANGE (gcd->slider),
-				     (double) status->volume);
-		g_free (status);
-	} else {
-		g_free (status);
-		gcd_warning ("Error getting status: %s", NULL);
+	/* Set the initial volume */
+	set_volume (gcd);
+	volume_notify_id = gconf_client_notify_add (client,
+			"/apps/gnome-cd/volume", volume_changed_cb,
+			gcd, NULL, &error);
+	if (error != NULL) {
+		g_warning ("Error: %s", error->message);
 	}
-
 	g_signal_connect (G_OBJECT (gcd->slider), "value-changed",
 			  G_CALLBACK (volume_changed), gcd);
 	
 	button_hbox = gtk_hbox_new (TRUE, 2);
-	
-	button = make_button_from_stock (gcd, GNOME_CD_PREVIOUS, G_CALLBACK (back_cb), _("Previous track"), _("Previous"));
-	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
-	gcd->back_b = button;
-
-	button = make_button_from_stock (gcd, GNOME_CD_REWIND, NULL, _("Rewind"), _("Rewind"));
-	g_signal_connect (G_OBJECT (button), "button-press-event",
-			  G_CALLBACK (rewind_press_cb), gcd);
-	g_signal_connect (G_OBJECT (button), "button-release-event",
-			  G_CALLBACK (rewind_release_cb), gcd);
-	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
-	gcd->rewind_b = button;
 
 	/* Create the play and pause images, and ref them so they never
 	   get destroyed */
-	gcd->play_image = gtk_image_new_from_stock (GNOME_CD_PLAY, GTK_ICON_SIZE_BUTTON);
+	gcd->play_image = gtk_image_new_from_stock (GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_BUTTON);
 	g_object_ref (gcd->play_image);
 
-	gcd->pause_image = gtk_image_new_from_stock (GNOME_CD_PAUSE, GTK_ICON_SIZE_BUTTON);
+	gcd->pause_image = gtk_image_new_from_stock (GTK_STOCK_MEDIA_PAUSE, GTK_ICON_SIZE_BUTTON);
 	gtk_widget_show (gcd->pause_image);
 	g_object_ref (gcd->pause_image);
 
@@ -752,12 +815,24 @@ init_player (const char *device_override)
 	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
 	gcd->play_b = button;
 	gcd->current_image = gcd->play_image;
-	
-	button = make_button_from_stock (gcd, GNOME_CD_STOP, G_CALLBACK (stop_cb), _("Stop"), _("Stop"));
+
+	button = make_button_from_stock (gcd, GTK_STOCK_MEDIA_STOP, G_CALLBACK (stop_cb), _("Stop"), _("Stop"));
 	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
 	gcd->stop_b = button;
 
-	button = make_button_from_stock (gcd, GNOME_CD_FFWD, NULL, _("Fast forward"), _("Fast forward"));
+	button = make_button_from_stock (gcd, GTK_STOCK_MEDIA_PREVIOUS, G_CALLBACK (back_cb), _("Previous track"), _("Previous"));
+	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
+	gcd->back_b = button;
+
+	button = make_button_from_stock (gcd, GTK_STOCK_MEDIA_REWIND, NULL, _("Rewind"), _("Rewind"));
+	g_signal_connect (G_OBJECT (button), "button-press-event",
+			  G_CALLBACK (rewind_press_cb), gcd);
+	g_signal_connect (G_OBJECT (button), "button-release-event",
+			  G_CALLBACK (rewind_release_cb), gcd);
+	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
+	gcd->rewind_b = button;
+	
+	button = make_button_from_stock (gcd, GTK_STOCK_MEDIA_FORWARD, NULL, _("Fast forward"), _("Fast forward"));
 	g_signal_connect (G_OBJECT (button), "button-press-event",
 			  G_CALLBACK (ffwd_press_cb), gcd);
 	g_signal_connect (G_OBJECT (button), "button-release-event",
@@ -765,10 +840,10 @@ init_player (const char *device_override)
 	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
 	gcd->ffwd_b = button;
 
-	button = make_button_from_stock (gcd, GNOME_CD_NEXT, G_CALLBACK (next_cb), _("Next track"), _("Next track"));
+	button = make_button_from_stock (gcd, GTK_STOCK_MEDIA_NEXT, G_CALLBACK (next_cb), _("Next track"), _("Next track"));
 	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
 	gcd->next_b = button;
-	
+
 	button = make_button_from_stock (gcd, GNOME_CD_EJECT, G_CALLBACK (eject_cb), _("Eject CD"), _("Eject"));
 	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
 	gcd->eject_b = button;
@@ -780,9 +855,23 @@ init_player (const char *device_override)
 
 	gcd->not_ready = FALSE;
 
+
+	tray_icon_create (gcd);
+	
+	return gcd;
+}
+
+void
+tray_icon_create (GnomeCD *gcd)
+{
+	GdkPixbuf *pixbuf;
+	GtkWidget *box;
+
 	/* Tray icon */
 	gcd->tray = GTK_WIDGET (egg_tray_icon_new ("GnomeCD Tray Icon"));
 	box = gtk_event_box_new ();
+	g_signal_connect (G_OBJECT (gcd->tray), "destroy",
+			 	G_CALLBACK (tray_icon_destroyed), gcd);
 	g_signal_connect (G_OBJECT (box), "button_press_event",
 			 	G_CALLBACK (tray_icon_clicked), gcd);
 	g_signal_connect (G_OBJECT (box), "key_press_event",
@@ -801,13 +890,11 @@ init_player (const char *device_override)
 	atk_object_set_name (gtk_widget_get_accessible (box), _("CD Player"));
 	gtk_container_add (GTK_CONTAINER (box), gcd->tray_icon);
 	gcd->tray_tips = gtk_tooltips_new ();
-	gtk_tooltips_set_tip (GTK_TOOLTIPS(gcd->tray_tips), box, _("CD Player"), NULL);
+	gtk_tooltips_set_tip (GTK_TOOLTIPS(gcd->tray_tips), gcd->tray, _("CD Player"), NULL);
 	g_signal_connect (G_OBJECT (box), "popup_menu",
 	            G_CALLBACK (popup_menu_cb), gcd);
 	
 	gtk_widget_show_all (gcd->tray);
-
-	return gcd;
 }
 
 static int 
@@ -838,13 +925,6 @@ static void client_die(GnomeClient *client,
 
 static const char *items [] =
 {
-	GNOME_CD_PLAY,
-	GNOME_CD_STOP,
-	GNOME_CD_PAUSE,
-	GNOME_CD_PREVIOUS,
-	GNOME_CD_NEXT,
-	GNOME_CD_FFWD,
-	GNOME_CD_REWIND,
 	GNOME_CD_EJECT
 };
 
@@ -895,6 +975,8 @@ main (int argc, char *argv[])
 		  N_("Only start if there isn't already a CD player application running"), NULL },
 		{ "play",   '\0', POPT_ARG_NONE, &cd_option_play, 0,
 		  N_("Play the CD on startup"), NULL },
+		{ "tray", '\0', POPT_ARG_NONE, &cd_option_tray, 0,
+		  N_("Start iconified to tray"), NULL},
 		{ NULL, '\0', 0, NULL, 0 },
 	};
 
@@ -948,7 +1030,9 @@ main (int argc, char *argv[])
 		exit (0);
 	}
 
-	gtk_widget_show_all (gcd->window);
+	if (!cd_option_tray)
+		gtk_widget_show_all (gcd->window);
+
 	g_signal_connect (client, "die",
 			  G_CALLBACK (client_die), gcd);
 
