@@ -12,13 +12,16 @@
 #include <config.h>
 #endif
 
+#define _ISOC99_SOURCE
+
 #include <string.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <unistd.h>
+#include <math.h>
 
 #include "gst-cdparanoia-cdrom.h"
 #include "cddb.h"
@@ -75,6 +78,8 @@ struct _GstCdparanoiaCDRomPrivate {
 
 	int check_playtime;
 	int end_playtime;
+
+	int volume;
 };
 
 static gboolean gst_cdparanoia_cdrom_eject (GnomeCDRom * cdrom,
@@ -102,6 +107,9 @@ static gboolean gst_cdparanoia_cdrom_get_status (GnomeCDRom * cdrom,
 						 status, GError ** error);
 static gboolean gst_cdparanoia_cdrom_close_tray (GnomeCDRom * cdrom,
 						 GError ** error);
+static gboolean gst_cdparanoia_cdrom_get_cddb_data (GnomeCDRom * cdrom,
+						GnomeCDRomCDDBData ** data,
+						GError ** error);
 
 static GnomeCDRomMSF blank_msf = { 0, 0, 0 };
 
@@ -205,12 +213,25 @@ eos (GstElement * element, gpointer data)
 	    (GstCdparanoiaCDRomPrivate *) data;*/
 }
 
+static gboolean
+stop_playback_idle_func (gpointer data) 
+{
+	GstCdparanoiaCDRomPrivate *priv =
+	    (GstCdparanoiaCDRomPrivate *) data;
+		    
+	gst_element_set_state (GST_ELEMENT (priv->play_thread),
+                               GST_STATE_NULL);
+	priv->check_playtime = 0;
+							  
+	return FALSE;
+}
+
 static void
 check_playtime (GstBin * bin, gpointer data)
 {
 	GstCdparanoiaCDRomPrivate *priv =
 	    (GstCdparanoiaCDRomPrivate *) data;
-	guint64 value = 0;
+	gint64 value = 0;
 	gboolean ret;
 	int i = 0;
 
@@ -219,9 +240,7 @@ check_playtime (GstBin * bin, gpointer data)
 	    gst_pad_query (GST_PAD (priv->cdp_pad), GST_QUERY_POSITION,
 			   &priv->sector_format, &value);
 	if (ret && priv->check_playtime && value >= priv->end_playtime) {
-		gst_element_set_state (GST_ELEMENT (priv->play_thread),
-				       GST_STATE_NULL);
-		priv->check_playtime = 0;
+		g_idle_add (stop_playback_idle_func, data);
 	} else {
 		for (i = 0; i < priv->number_tracks; i++) {
 			if (value <
@@ -241,8 +260,8 @@ cb_error (GstElement *el, GstElement *src,
 	g_warning (err->message);
 }
 
-static void
-build_pipeline (GstCdparanoiaCDRom * lcd)
+static gboolean
+build_pipeline (GstCdparanoiaCDRom * lcd, GError ** error)
 {
 	GstCdparanoiaCDRomPrivate *priv;
 	static int pipeline_built = 0;
@@ -251,7 +270,7 @@ build_pipeline (GstCdparanoiaCDRom * lcd)
 	GstElement *conv, *scale, *vol, *pthread, *queue;
 
 	if (pipeline_built == 1)
-		return; 
+		return TRUE; 
 
 	priv = lcd->priv;
 
@@ -266,7 +285,11 @@ build_pipeline (GstCdparanoiaCDRom * lcd)
 
 	priv->cdparanoia =
 	    gst_element_factory_make ("cdparanoia", "cdparanoia");
-	g_assert (priv->cdparanoia != 0);	/* TBD: GError */
+	if (!priv->cdparanoia) {
+		g_set_error (error, 0, 0,
+			     _("Failed to set up CD reader; please install the GStreamer cdparanoia plugin."));
+		return FALSE;
+	}
 
 	if (priv->cd_device) {
 		if (g_object_class_find_property (
@@ -288,13 +311,19 @@ build_pipeline (GstCdparanoiaCDRom * lcd)
 	g_assert (priv->cdp_pad != 0);	/* TBD: GError */
 
 	priv->audio_sink = gst_gconf_get_default_audio_sink ();
-	g_assert (priv->audio_sink != 0);	/* TBD: GError */
+	if (!priv->audio_sink) {
+		g_set_error (error, 0, 0,
+			     _("Failed to set up audio output; please install a GStreamer plugin for your soundcard."));
+		return FALSE;
+	}
 
 	pthread = gst_element_factory_make ("thread", "outthr");
 	queue = gst_element_factory_make ("queue", "q");
 	conv = gst_element_factory_make ("audioconvert", "conv");
 	scale = gst_element_factory_make ("audioscale", "scale");
 	priv->vol_element = vol = gst_element_factory_make ("volume", "vol");
+	g_object_set (G_OBJECT (priv->vol_element), "volume",
+		      (gdouble) priv->volume / 255.0, NULL);
 
 	/* Now build the pipeline */
 	gst_bin_add_many (GST_BIN (pthread), queue, conv,
@@ -307,6 +336,8 @@ build_pipeline (GstCdparanoiaCDRom * lcd)
 			       priv->audio_sink, NULL);
 
 	pipeline_built = 1;
+
+	return TRUE;
 }
 
 
@@ -887,7 +918,9 @@ gst_cdparanoia_cdrom_play (GnomeCDRom * cdrom,
 		}
 
 		/* PLAY IT AGAIN */
-		build_pipeline (lcd);
+		if (!build_pipeline (lcd, error))
+			return FALSE;
+
 		if (GST_STATE (priv->play_thread) <= GST_STATE_PAUSED) {
 			gst_element_set_state (GST_ELEMENT (priv->play_thread),
 					       GST_STATE_PAUSED);
@@ -1144,6 +1177,7 @@ gst_cdparanoia_cdrom_get_status (GnomeCDRom * cdrom,
 	GstCdparanoiaCDRom *lcd;
 	GstCdparanoiaCDRomPrivate *priv;
 	GnomeCDRomStatus *realstatus;
+	GnomeCDRomCDDBData *data;	
 #if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
 	struct cd_sub_channel_position_data subchnl;
 #else
@@ -1151,7 +1185,7 @@ gst_cdparanoia_cdrom_get_status (GnomeCDRom * cdrom,
 #endif
 	int cur_gst_status;
 	int cd_status;
-	guint64 value = 0;
+	gint64 value = 0;
 	gboolean ret;
 
 	g_return_val_if_fail (status != NULL, TRUE);
@@ -1208,7 +1242,11 @@ gst_cdparanoia_cdrom_get_status (GnomeCDRom * cdrom,
 			return TRUE;
 
 		default:
-			realstatus->cd = GNOME_CDROM_STATUS_OK;
+			gst_cdparanoia_cdrom_get_cddb_data (cdrom, &data, NULL);
+			if (data == NULL)
+				realstatus->cd = GNOME_CDROM_STATUS_EMPTY_DISC;
+			else
+				realstatus->cd = GNOME_CDROM_STATUS_OK;
 			break;
 		}
 	} else {
@@ -1239,10 +1277,11 @@ gst_cdparanoia_cdrom_get_status (GnomeCDRom * cdrom,
 	   happens that finds the CD caught up. cur_track is an answer to
 	   that where we keep better track of what's currently playing than
 	   the hardware. */
+	if (!build_pipeline (lcd, error))
+		return FALSE;
 
-	build_pipeline (lcd);
 	g_object_get (G_OBJECT (priv->vol_element), "volume", &vol, NULL);
-	realstatus->volume = lrint (vol * 255.0);
+	realstatus->volume = priv->volume = rint (vol * 255.0);
 	cur_gst_status =
 	    gst_element_get_state (GST_ELEMENT (priv->play_thread));
 
@@ -1402,6 +1441,7 @@ gst_cdparanoia_cdrom_set_volume (GnomeCDRom * cdrom,
 
 	g_object_set (G_OBJECT (priv->vol_element), "volume",
 		      (gdouble) volume / 255.0, NULL);
+	priv->volume = volume;
 
 	return TRUE;
 }
@@ -1504,7 +1544,7 @@ gst_cdparanoia_cdrom_get_cddb_data (GnomeCDRom * cdrom,
 	(*data)->nsecs =
 	    (priv->track_info[priv->track1].address.minute * 60) +
 	    priv->track_info[priv->track1].address.second;
-	(*data)->offsets = g_new0 (unsigned int, priv->track1 + 1);
+	(*data)->offsets = g_new0 (int, priv->track1 + 1);
 
 	for (i = priv->track0 - 1; i < priv->track1; i++) {
 		(*data)->offsets[i] =
